//+++++++++++++++++++++++++++++Ambilight PWM++++++++++++++++++++++++++++++

//Calculates the color sequence based on the time

void ambilight_pwm_calc(void)
{
    static uint8_t prescaler_blue, prescaler_green;
    uint8_t rgbtable_value_blue, rgbtable_value_green, rgbtable_value_red;
    //Control color by time
    if (effect_ctrl.light == FALSE)
        {
        //red (once a minute)
        if (pwm_sequence.red == FALSE)
            {
            pwm_red_count++;
            pwm_red = pgm_read_byte (rgbtable_time_p + pwm_red_count);
            if (pwm_red_count == PWM_MAX)
                pwm_sequence.red = TRUE;
            }
            //dim down
        else 
            {
            pwm_red_count--;
            pwm_red = pgm_read_byte (rgbtable_time_p + pwm_red_count);
            if (pwm_red_count == PWM_MIN)
                pwm_sequence.red = FALSE;
            }

        //green (once a hour)
        prescaler_green++;
        if (prescaler_green == 60)
            {
            prescaler_green = 0;
            //dim up
            if (pwm_sequence.green == FALSE)
                {
                pwm_green_count++;
                pwm_green = pgm_read_byte (rgbtable_time_p + pwm_green_count);
                if (pwm_green_count == PWM_MAX)
                    pwm_sequence.green = TRUE;
                }
            //dim down
            else 
                {
                pwm_green_count--;
                pwm_green = pgm_read_byte (rgbtable_time_p + pwm_green_count);
                if (pwm_green_count == PWM_MIN)
                    pwm_sequence.green = FALSE;
                }

            //blue (once a day)
            prescaler_blue++;
            if (prescaler_blue == 24)
                {
                prescaler_blue = 0;
                //dim up
                if (pwm_sequence.blue == FALSE)
                    {
                    pwm_blue_count++;
                    pwm_blue = pgm_read_byte (rgbtable_time_p + pwm_blue_count);
                    if (pwm_blue_count == PWM_MAX)
                        pwm_sequence.blue = TRUE;
                    }
                    //dim down
                else 
                    {
                    pwm_blue_count--;
                    pwm_blue = pgm_read_byte (rgbtable_time_p + pwm_blue_count);
                    if (pwm_blue_count == PWM_MIN)
                        pwm_sequence.blue = FALSE;
                    }
                }
            }
        }
    //Control color by temperature
    else
        {
        //between 0 and 40 degrees read RGB values from lookup tables
        //values over 40 degrees are not stored to save flash memory.
            
        if (rgb_temperature <= 40)
            {
            rgbtable_value_blue = (uint8_t) pgm_read_byte(rgbtable_blue_p + rgb_temperature);
            rgbtable_value_green = (uint8_t) pgm_read_byte(rgbtable_green_p + rgb_temperature);
            rgbtable_value_red = (uint8_t) pgm_read_byte(rgbtable_red_p + rgb_temperature);
                
            //overblending for temp change
                if (pwm_blue == rgbtable_value_blue)
                    asm("NOP"); //nothing to do in this case, just to fasten the function a bit
                else if (pwm_blue < rgbtable_value_blue  && pwm_blue < PWM_MAX)
                    pwm_blue++;
                else if (pwm_blue > rgbtable_value_blue  && pwm_blue > PWM_MIN)
                    pwm_blue--;
 
                if (pwm_green == rgbtable_value_green)
                    asm("NOP"); //nothing to do in this case, just to fasten the function a bit
                else if (pwm_green < rgbtable_value_green  && pwm_green < PWM_MAX)
                    pwm_green++;
                else if (pwm_green > rgbtable_value_green  && pwm_green > PWM_MIN)
                    pwm_green--;

                if (pwm_red == rgbtable_value_red)
                    asm("NOP"); //nothing to do in this case, just to fasten the function a bit
                else if (pwm_red < rgbtable_value_red  && pwm_red < PWM_MAX)
                    pwm_red++;
                else if (pwm_red > rgbtable_value_red  && pwm_red > PWM_MIN)
                    pwm_red--;
            }
        else
            {
            pwm_blue=PWM_MIN;
            pwm_green=PWM_MIN;
            pwm_red=PWM_MAX;
            }
        }
}

//this is the pwm for the ambilight

void ambilight_pwm(void)
{
    pwmdiv++;
    if (pwmdiv == PWM_DIVIDER)
	pwmdiv = 0;
    else
	{
	//red
	if (pwmdiv > pwm_red)
		PORTC &= ~_BV(LED2);
	else 
		PORTC |= _BV(LED2);
	//green
	if (pwmdiv > pwm_green)
		PORTD &= ~_BV(LED3);
	else 
		PORTD |= _BV(LED3);
	//blue 
	if (pwmdiv > pwm_blue)
		PORTC &= ~_BV(LED4);
	else 
		PORTC |= _BV(LED4);
	}
}

//This recreates the pwm_sequence start colors

void recreate_pwm_sequence(void)
{
    //Color is controlled by time
    if (effect_ctrl.light == FALSE) 
        {
        if (time.h < 12)
            {
            //calculate: hours * 10 + part of minutes (max. 5) + part of seconds (max 5) + 1 = 1...121
            pwm_blue_count = (time.h * 10  + time.m / 10 + time.s / 10) + 1;
            pwm_blue = (uint8_t) pgm_read_byte (rgbtable_time_p + pwm_blue_count);
            pwm_sequence.blue = FALSE;
            }
        else
            {
            //calculate: PWM_MAX - ((hours - 12) * 10 + part of minutes (max. 5) + part of seconds (max 5)) = 121...1
            pwm_blue_count = PWM_MAX - ((time.h - 12) * 10  + time.m / 10 + time.s / 10);
            pwm_blue = (uint8_t) pgm_read_byte (rgbtable_time_p + pwm_blue_count);
            pwm_sequence.blue = TRUE;
            }
        if (time.m < 30)
            {
            //calculate: (minutes * 4 + part of seconds (max. 4)) = 1...121
            pwm_green_count = (time.m * 4  + time.s / 15) + 1;
            pwm_green = (uint8_t) pgm_read_byte (rgbtable_time_p + pwm_green_count);
            pwm_sequence.green = FALSE;
            }
        else
            {
            //calculate: (PWM_MAX - ((minutes - 30) * 4 + part of seconds (max 4)) = 121...1
            pwm_green_count = PWM_MAX - ((time.m - 30) * 4  + time.s / 15);
            pwm_green = (uint8_t) pgm_read_byte (rgbtable_time_p + pwm_green_count);
            pwm_sequence.green = TRUE;
            }
        if (time.s <= 30)
            {
            //calculate: seconds * 4 + 1 = 1...121
            pwm_red_count = time.s * 4 + 1;
            pwm_red = (uint8_t) pgm_read_byte (rgbtable_time_p + pwm_red_count);
            pwm_sequence.red = FALSE;
            }
        else
            {
            //calculate: PWM_MAX - ((seconds - 30) * 4) - correcture value (max 4) = 121...1
            pwm_red_count = PWM_MAX - ((time.s - 30) * 4) - ((time.s - 30) / 7);
            pwm_red = (uint8_t) pgm_read_byte (rgbtable_time_p + pwm_red_count);
            pwm_sequence.red = TRUE;
            }
        }
    //Color is controlled by temperature
    else 
        {
        pwm_blue = (uint8_t) pgm_read_byte(rgbtable_blue_p + rgb_temperature);
        pwm_green = (uint8_t) pgm_read_byte(rgbtable_green_p + rgb_temperature);
        pwm_red = (uint8_t) pgm_read_byte(rgbtable_red_p + rgb_temperature);
        }
} 
