/****************************TEMPERATURE***************************/

// Read out temperature from ADC and build average if requested.
// for sort algo's see https://openbook.rheinwerk-verlag.de/c_von_a_bis_z/022_c_algorithmen_003.htm

void read_temperature(void)
{
    uint16_t temp;
    uint8_t i,j,pos;
    
    for (i=0;i<ADC_READINGS;i++)
        {
        ADCSRA |= (1<<ADSC);
        //read out adc
        while (ADCSRA & (1<<ADSC))
            {
            }
        if ((ADC+SENSOROFFSET) > SENSORNULL && ADC <= SENSOR100)
            temp_avg[i]=ADC;
        else
            {
            //Sensor reading is out of limits, so no sensor is connected or reading is unreliable.
            temperature = 0;
            rgb_temperature = 0;
            return;
            }
        }
        
    //Modified median average of ADC readings with selection sort.
    //Used to reduce noise, remove spikes etc.
    //The sort algorithm will sort the array so the most low and high values are located on the arrays borders,
    //while the middle of the array contains the average.
    //After sort the value in the middle of the array will be used for further calculation.
    //Instead of averaging a few values from the middle of the array like in normal median, we will just keep 1 value per measurement cycle and 
    //average the values of TEMP_AVG_DEEP measurement cycles before displaying it.
    
    for(i=0;i<ADC_READINGS;i++)
        {
        pos = i;
        for(j=i+1;j<ADC_READINGS;j++)
            {
            if (temp_avg[j] < temp_avg[pos])
                pos = j;
            }
        if (pos != i)
            {
            temp = temp_avg[pos];
            temp_avg[pos] = temp_avg[i];
            temp_avg[i] = temp;
            }
        }

    //Average values from TEMP_AVG_DEEP measurement cycles.

    temp_calc+=temp_avg[ADC_READINGS/2];
    avg_count++;
    if (avg_count == TEMP_AVG_DEEP)
        {
        avg_count = 0;
        temp_calc = temp_calc / TEMP_AVG_DEEP;
    
        //correction via temperature offset
        if (tempoffset <= 20 && temp_calc > 20)
            temp_calc = temp_calc - (20 - tempoffset);
        else
            temp_calc = temp_calc + (tempoffset - 20);
    
        //simple calculation for temp including correction factors.
        //1000 100.0 degress spreading without "." 
        temp_calc = ((temp_calc+SENSOROFFSET)-SENSORNULL)*1000/(SENSOR100-SENSORNULL);
    
        temperature = temp_calc;
        temp_calc = 0;
        rgb_temperature = temperature / 10;
        }
}


